import { Injectable, HttpException, HttpStatus } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';

import { MapDrawingEntity } from '../draw/coordinate.entity';
import { MapDrawingDTO } from './mapdrawing.dto';
import { NotificationGateway } from 'src/notifications/notifications.gateway';

@Injectable()
export class DrawService {
  constructor(
    @InjectRepository(MapDrawingEntity)
    private mapDrawingRepository: Repository<MapDrawingEntity>,
    private notificationGateway: NotificationGateway,
  ) {}

  async draw(gameperson, gameId, data: MapDrawingDTO) {
    data['gameId'] = gameId;
    const drawing = await this.mapDrawingRepository.create(data);
    this.notificationGateway.server.emit(gameId, {
      type: 'drawing-update',
    });
    // create new instance if id is null
    if (data.mapDrawingId == null || data.mapDrawingId == '') {
      drawing.faction = gameperson.faction;
      console.log(drawing);
      const mapDrawing = await this.mapDrawingRepository.insert(drawing);
      return mapDrawing.identifiers;
    }
    // get ref from db
    const draw = await this.mapDrawingRepository.findOne({
      where: { mapDrawingId: data.mapDrawingId },
      relations: ['faction'],
    });
    if (await draw.ownershipCheck(gameperson.faction, gameperson.role)) {
      // else update the existing instance
      return await this.mapDrawingRepository.save(drawing);
    }

    throw new HttpException(
      'Drawing is not from your faction!',
      HttpStatus.BAD_REQUEST,
    );
  }

  // draw map based on game and gameperson faction
  async drawMap(gameperson, gameId) {
    // return all active drawings if admin
    if (gameperson.role === 'admin') {
      return await this.mapDrawingRepository.find({
        where: { gameId: gameId, drawingIsActive: true },
        relations: ['faction'],
      });
    }
    // return mapdrawings with given faction and gameid
    return await this.mapDrawingRepository.find({
      where: [
        {
          gameId: gameId,
          faction: gameperson.faction,
          drawingIsActive: true,
        },
        {
          gameId: gameId,
          faction: null,
          drawingIsActive: true,
        },
      ],
      relations: ['faction'],
    });
  }
}
